#!/usr/bin/perl -w

# buildtables-v0.06.pl by Randy Nash (nashr@atriskonline.com)
# Copyright (C) 2004-2005 Randy Nash - @RISK Online (www.atriskonline.com)
#
# This code builds all the tables necessary to track traffic statistics.  It
# is part of the iptraffic application.
#
# --- GNU General Public License Disclamer ---
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

use strict;
use DBI;

# Connect To Database
my $database = "iptraffic";
my $hostname = "localhost";
my $username = "root";
my $password = "";
my $dbh = DBI->connect("DBI:mysql:$database:$hostname", $username, $password);
die "Cannot log into database.  Please try again later.\n" unless $dbh;
print "Connected to $database... \n";

# All of the fields of an ethernet packet
my $sql = qq{CREATE TABLE etherpkt	(
			pid		INT		UNSIGNED	NOT NULL	AUTO_INCREMENT,
			pkt_ts		timestamp,
			pkt_proto	char(8),
			eth_type	MEDIUMINT	UNSIGNED,
			eth_srcmac	char(12),
			eth_destmac	char(12),
			eth_data	TEXT,
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);

# All of the fields of an arp packet
my $sql = qq{CREATE TABLE arppkt	(
			pid		INT		UNSIGNED	NOT NULL,
			arp_htype	TINYINT		UNSIGNED	NOT NULL,
			arp_proto	TINYINT   	UNSIGNED	NOT NULL,
			arp_hlen	SMALLINT	UNSIGNED,
			arp_plen	SMALLINT	UNSIGNED,
			arp_opcode	TINYINT		UNSIGNED	NOT NULL,
			arp_sha		char(12),
			arp_spa		char(12),
			arp_tha		char(12),
			arp_tpa		char(12),
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);



# All of the fields of an igmp packet
my $sql = qq{CREATE TABLE igmppkt	(
			pid		INT		UNSIGNED	NOT NULL,
			igmp_ver	TINYINT		UNSIGNED	NOT NULL,
			igmp_type	TINYINT		UNSIGNED	NOT NULL,
			igmp_len	TINYINT		UNSIGNED	NOT NULL,
			igmp_subtype	SMALLINT	UNSIGNED,
			igmp_cksum	SMALLINT	UNSIGNED,
			igmp_group_addr	TINYINT		UNSIGNED	NOT NULL,
			igmp_data	TEXT,
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);

# All of the fields of an stp packet
my $sql = qq{CREATE TABLE stppkt	(
			pid			INT		UNSIGNED	NOT NULL,
			stp_max_age		INT		UNSIGNED,
			stp_message_age		INT		UNSIGNED,
			stp_bpdu_flags		INT		UNSIGNED,
			stp_bridge_mac		INT		UNSIGNED,
			stp_bpdu_type		INT		UNSIGNED,
			stp_topology_change	INT		UNSIGNED,
			stp_bridge_priority	INT		UNSIGNED,
			stp_topology_change_ack	INT		UNSIGNED,
			stp_protocol_version	INT		UNSIGNED,
			stp_forward_delay	INT		UNSIGNED,
			stp_hello_time		INT		UNSIGNED,
			stp_port_num		INT		UNSIGNED,
			stp_root_priority	INT		UNSIGNED,
			stp_root_path_cost	INT		UNSIGNED,
			stp_protocol_id		INT		UNSIGNED,
			stp_root_mac		INT		UNSIGNED,
			stp_port_priority	INT		UNSIGNED,
			stp_root_id		INT		UNSIGNED,
			stp_port_id		TEXT,
			stp_bridge_id		INT		UNSIGNED,
			PRIMARY KEY		(pid)
			)};
$dbh->do($sql);

# All of the fields of an ip packet
my $sql = qq{CREATE TABLE ippkt	(
			pid		INT		UNSIGNED	NOT NULL,
			ip_ver		TINYINT		UNSIGNED,
			ip_hlen		TINYINT		UNSIGNED,
			ip_flags	TINYINT   	UNSIGNED,
			ip_foffset	SMALLINT  	UNSIGNED,
			ip_tos		TINYINT		UNSIGNED,
			ip_len		SMALLINT	UNSIGNED,
			ip_id		SMALLINT  	UNSIGNED,
			ip_ttl		TINYINT   	UNSIGNED,
			ip_proto	TINYINT   	UNSIGNED	NOT NULL,
			ip_cksum	SMALLINT  	UNSIGNED,
			ip_srcip	INT		UNSIGNED,
			ip_dstip	INT		UNSIGNED,
			ip_data		TEXT,
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);

# All of the fields of an tcp packet
my $sql = qq{CREATE TABLE tcppkt	(
			pid		INT		UNSIGNED	NOT NULL,
			tcp_sport	SMALLINT	UNSIGNED	NOT NULL,
			tcp_dport	SMALLINT	UNSIGNED	NOT NULL,
			tcp_seqnum	INT		UNSIGNED,
			tcp_acknum	INT		UNSIGNED,
			tcp_hlen	TINYINT		UNSIGNED,
			tcp_res		TINYINT		UNSIGNED,
			tcp_flags	TINYINT		UNSIGNED	NOT NULL,
			tcp_win		SMALLINT	UNSIGNED,
			tcp_cksum	SMALLINT	UNSIGNED,
			tcp_urg		SMALLINT	UNSIGNED,
			tcp_data	TEXT,
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);

# All of the fields of an udp packet
my $sql = qq{CREATE TABLE udppkt	(
			pid		INT		UNSIGNED	NOT NULL,
			udp_sport	SMALLINT	UNSIGNED	NOT NULL,
			udp_dport	SMALLINT	UNSIGNED	NOT NULL,
			udp_len		SMALLINT	UNSIGNED,
			udp_cksum	SMALLINT	UNSIGNED,
			udp_data	TEXT,
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);

# All of the fields of an icmp packet
my $sql = qq{CREATE TABLE icmppkt	(
			pid		INT		UNSIGNED	NOT NULL,
			icmp_type	TINYINT		UNSIGNED	NOT NULL,
			icmp_code	TINYINT		UNSIGNED	NOT NULL,
			icmp_cksum	SMALLINT	UNSIGNED,
			icmp_data	TEXT,
			PRIMARY KEY	(pid)
			)};
$dbh->do($sql);

$dbh->disconnect();

